function gtCopyCorrectUndistortCondor(~, ~, workingdirectory)
% GTCOPYCORRECTUNDISTORTCONDOR  Handles the parallelisation
%     gtCopyCorrectUndistortCondor(first, last, workingdirectory)
%     -----------------------------------------------------------
%     Version 002 26-11-2011 by LNervo
%       Comments and clean version.
%     Sub-functions:
%[sub]- sfRename
%
%     Version 001 by AKing
%       gtCopy does the business of copying/correcting/undistorting
%       talking to the database to
%       coordinate copying between multiple condor jobs.
%
%       all filenames, directory names, and paths are determined from the
%       parameters file
%
%       first and last are dummy variables - not used by the function.
%

if isdeployed
    global GT_MATLAB_HOME
    global GT_DB
    load('workspaceGlobal.mat')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   load and read data from parameters.mat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd(workingdirectory)
parameters = [];
load('parameters.mat');

source_dir    = parameters.acq.collection_dir;
dest_dir      = fullfile(parameters.acq.dir, '0_rawdata', parameters.acq.name);
dest_dir_orig = fullfile(parameters.acq.dir, '0_rawdata', 'Orig');

%database connect
gtDBConnect('graindb.esrf.fr', 'gtadmin', 'gtadmin', 'graintracking');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   check directories are okay
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (~exist(source_dir, 'dir'))
    gtError('Cannot find source directory!')
end
% These two directory creations should be redundant
if (~exist(dest_dir, 'dir')), mkdir(dest_dir); end
if (~exist(dest_dir_orig, 'dir')), mkdir(dest_dir_orig); end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   has the data already been moved to analysis directory?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% determine if we need to move the data from the collection directory to the analysis location.
% If sourcedir is "xxx/0_rawdata/Orig", and destinationdir is "xxx/0_rawdata/scanname"
% then data has already been moved (ie sourcedir==destdir_orig)
%
% directory formats are
% either
% /data/id19/graintracking/ ..path.. /scanname/
% or
% /mntdirect/_data_id19_graintracking/ ..path../scanname/
%
% therefore need two tests

% are paths the same?
% what is the source dir name?
[path1, source_dirname, ~] = fileparts(source_dir);
[path2,              ~, ~] = fileparts(dest_dir);

move_data_flag = ~(strcmp(path1, path2) || strcmp(source_dirname, 'Orig'));
if (move_data_flag)
    disp('Not moving data - it is already in the analysis location')
else
    disp('Moving (or copying) data to the analysis location')
end

filetable = [parameters.acq.name '_filetable'];
base_query = sprintf('insert into %s (filename) values ("%%s")', filetable);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   copying routine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Copying raw images...')
counter = 0;
should_stop = false;
% while loop for function timeout when copying is finished
while (~should_stop)
    found = false;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   get the list of files to try and copy
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % source directory file list
    src_files = dir(fullfile(source_dir, '*'));
    copied_files = dir(fullfile(dest_dir, '*'));

    % go through the source list, remove directories, parameters.mat,
    filter_names = [{copied_files.('name')}, 'parameters.mat'];
    files_to_keep = ~ismember({src_files(:).name}, filter_names);
    files_to_keep = files_to_keep & ~[src_files(:).isdir];
    src_files = src_files(files_to_keep);

    % sort files by date order to copy in order
    [~, indx] = sort( [src_files.('datenum')] );

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   try to copy files
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for n = reshape(indx, [1 numel(indx)])
        sfname = src_files(n).name;
        query = sprintf(base_query, sfname);

        %check if sfname has already been taken by another condor job
        try
            test = mym(query);
            % Andy 18/03/2012 - fixed this: job should not die if try fails,
            % but test should be false
        catch mexc
            if (strncmpi(mexc.message, 'duplicate', 9))
                fprintf('%s, skipping...\n', mexc.message)
                test = false;
            else
                % If not a duplicate entry, this is a more serious error!
                rethrow(mexc)
            end
        end

        if (test) % no other job is copying this file, do copy
            disp(['working on ' sfname])
            fname_in = fullfile(source_dir, sfname);
            fname_out_orig = fullfile(dest_dir_orig, sfname);

            if (isfield(parameters.acq, 'interlaced_turns') ...
                    && (parameters.acq.interlaced_turns > 0) )
                %if using interlaced turns, may need some renaming
                try
                    % determine how file should be renamed
                    newfname = edf_rename(sfname, parameters);
                    fnameout = fullfile(dest_dir, newfname);
                catch mexc
                    gtPrintException(mexc)
                    fnameout = fullfile(dest_dir, sfname);
                end
            else % standard routine
                fnameout = fullfile(dest_dir, sfname);
            end

            try
                gtCopy(fname_in, fnameout, fname_out_orig, parameters, move_data_flag);
                found = true;
            catch mexc
                if gtCheckExceptionType(mexc, 'EDF:')
                    message = [ 'Skipping badly formed file: "' fname_in ...
                        '" (' mexc.identifier ')'];
                    gtPrintException(mexc, message);
                else
                    gtPrintException(mexc, 'Error in the code!!');
                    rethrow(mexc)
                end
            end
        end
    end %for loop over the file list, working in date order

    % Checking for stopping conditions
    if (isfield(parameters.acq, 'online') && parameters.acq.online)
        if (~found)
            counter = counter + 1;
            pause(10) % have a rest, wait before looking again
            disp('sleeping 10, then look again')
        else
            counter = 0;
        end

        % Check if we waited more than 16 minutes for new images that didn't
        % come
        should_stop = counter >= 100;
    else
        should_stop = true;
    end
end
disp('Timed out while waited for files to appear... quitting')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% sfCopy was a single subfunction in several scripts - I have created a
% single gtCopy function to be used in all cases
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function newname = edf_rename(oldname, parameters)
% GTCOPYCORRECTUNDISTORTCONDOR/SFRENAME Determines how files should be renamed
% Name convention, description of scheme:
%
% for 80 images collected over two turns
% note this would be nproj=40 in ftomosetup, so be careful!
% keep parameters.acq.nproj as images/180degrees in final data
%
% first turn:
% ref0000_0000.edf..., name_0_0000.edf, name_0_0001.edf... name_0_0039.edf,
% ref0000_0040.edf
%
% second turn: (This was probably true in the past)
% name_1_0000.edf, name_1_0001.edf... name_1_0039.edf, ref0000_1_0040.edf
% second turn: (This is TRUE now)
% name_1_0000.edf, name_1_0001.edf... name_1_0039.edf, ref0000_0040_1.edf

    [~, fname, ext] = fileparts(oldname);

    if (strcmp(ext, '.edf') && ~(strncmpi(fname, 'dark', 4)))
        % we want sequentially number filenames and references groups, without
        % the turn indication.
        if (strncmpi(fname, 'ref', 3) || strncmpi(fname, 'quali', 5))
            splitted_name = regexp(fname, '_', 'split');
            % bad idea to name scans ref* or quali*!
            switch (numel(splitted_name))
                case 3
                    chief_turn = str2double(splitted_name{end});
                    old_number = str2double(splitted_name{end-1});
                case 2
                    chief_turn = 0;
                    old_number = str2double(splitted_name{end});
                otherwise
                    % don't rename / renumber / overwrite if not sure!
                    gtError('gtCopyCorrectUndistortCondor:wrong_renaming', ...
                        'dont recognise the name of this edf!')
            end
            % keep the ref00xx_ and quali_ part of the name
            base_name = [splitted_name{1} '_'];

        elseif (strncmpi(fname, parameters.acq.name, length(parameters.acq.name)))
            % We remove the assumption of '_' at the end of the acquisition name
            numbers_name = strrep(fname, parameters.acq.name, '');
            splitted_name = regexp(numbers_name, '_', 'split');
            % if this is an image.
            if (parameters.acq.interlaced_turns > 0)
                chief_turn = str2double(splitted_name{end-1});
            else
                chief_turn = 0;
            end
            old_number = str2double(splitted_name{end});
            base_name = parameters.acq.name;
        else
            % don't rename / renumber / overwrite if not sure!
            gtError('gtCopyCorrectUndistortCondor:wrong_renaming', ...
                'dont recognise the name of this edf!')
        end
        images_per_turn = (2 * parameters.acq.nproj) ...
            / (parameters.acq.interlaced_turns + 1);
        new_number = old_number + (chief_turn * images_per_turn);
        % note that in some datasets we exceed 9999 images.
        newname = sprintf('%s%04d.edf', base_name, new_number);
    else
        % Either not an edf file or darks
        newname = oldname;
    end
end
