function [poles, cmap, valids] = gtCrystAxisPoles(phaseid, pl_norm, r_vectors, save_flag)
% GTCRYSTAXISPOLES  Makes a colourmap according to a crystallographic axis
%                   orientation and gives the colormap.
%     [poles, cmap, valids] = gtCrystAxisPoles(phaseid, pl_norm, r_vectors, save_flag)
%     --------------------------------------------------------------------------------
%     Calculates poles wrt a crystallographic direction using r_vectors.
%     Loads parameters.mat:spacegroup, latticepar, crystal_system
%
%     INPUT:
%       phaseid   = <int>        phase number {1}
%       pl_norm   = <double>     hkil of the plane normal wrt calculate the poles (1x4) 
%                                {[0 0 0 1],},[2 -1 -1 0], ...
%       r_vectors = <double>     Rodrigues vectors (Nx3)
%       save_flag = <logical>    flag to save file {false}
%
%     OUTPUT:
%       poles     = <double>     list of poles direction of all the grains (Mx4)
%                                We keep the index of the reference r_vector (first column)
%                                NB. M is different from N because of the crystal symmetry.
%       cmap      = <double>     poles colourmap (Nx4)
%       valids    = <double>     poles list (only one per orientation) (Nx3)
%
%     Version 004 10-01-2013 by LNervo
%       Bug fix with choice of positive poles; choice of a pole for the grain
%       colour
%
%     Version 003 19-10-2012 by YGuilhem
%       Factorized using gtVectorOrientationColor
%
%     Version 002 28-06-2012 by LNervo
%       Update to version 2 of parameters; cleaning formatting


if ~exist('phaseid','var') || isempty(phaseid)
    phaseid = 1;
end
if ~exist('pl_norm','var') || isempty(pl_norm)
    pl_norm = [0 0 0 1];
end
if ~exist('r_vectors','var') || isempty(r_vectors)
    gtError('gtCrystAxisPoles:missingArgument','The argument r_vectors is empty...')
end
if ~exist('save_flag','var') || isempty(save_flag)
    save_flag = false;
end
  
pl_norm_str = num2str(pl_norm);
pl_norm_str = strrep(pl_norm_str,' ','');

parameters = [];
load('parameters.mat');
crystal_system = parameters.cryst(phaseid).crystal_system;
spacegroup = parameters.cryst(phaseid).spacegroup;
latticepar = parameters.cryst(phaseid).latticepar;

if size(r_vectors,2) == 4
    r_vectors = r_vectors(:,2:4);
end

% take all the symmetric equivalents
all_hkils = gtGetReflections(pl_norm, spacegroup);
% finds the original pl_norm among the list of its symmetries
[~,ind,~] = findDifferentRowIntoMatrix(all_hkils, pl_norm);
% going to cartesian reciprocal space + normalisation of the cartesian space
normalised_hkls = gtHex2Cart(all_hkils, latticepar);

poles = zeros(0,4);
valid_poles = zeros(0,4);

% Compute all orientation matrices g
all_g = gtMathsRod2OriMat(r_vectors.');

% Express normalised hkls in cartesian SAMPLE CS
all_normals = gtVectorCryst2Lab(normalised_hkls, all_g);

for ii = 1:length(all_g)
    normals = all_normals(:, :, ii);
    for jj = 1:size(all_normals, 1)
        % takes the positive poles and saves the grainid which it refers to
        if (normals(jj, 3) >= 0)
            poles(end+1, 1) = ii;
            poles(end, 2:4) = normals(jj, :);
        else
            poles(end+1, 1) = ii;
            poles(end, 2:4) = -normals(jj, :);
        end
        % hard coded: among the symmetries takes the original pl_norm
        if (ind == jj)
            valid_poles(end+1, :) = poles(end, :);
        end
    end
end

% sorts the poles by rows and removes the duplicates
tmp = sortrows(poles,[1 2]);
poles = unique(tmp,'rows');
clear tmp

if nargout > 1
    cmap = gtCrystAxisCmap(valid_poles);
    if nargout > 2
        valids = valid_poles;
    end
end


if save_flag
    % saves the list of poles to be used in the pole figure
    fname = fullfile('6_rendering',['poles_' pl_norm_str '.mat']);
    save(fname,'poles'); % list of the poles
    disp(['Saved poles list in ' fname])

    % calculates the colour map using the valid poles list : only one colour for each
    % grain
    % Save the colourmap
    fname = fullfile('6_rendering',['poles_cmap_' pl_norm_str '.mat']);
    save(fname,'cmap'); % list of the poles
    disp(['Saved colormap according to the poles list in ' fname])
else
    disp('Nothing has been save don disk yet')
end

end % end of function
