function [f_handle, p_handle] = gtDrawGrainUnitCells(grains, varargin)
%     [f_handle, p_handle] = gtDrawGrainUnitCells(grains, varargin)
%     -------------------------------------------------------------
%     Generalized cases to plot more than one dataset.
%
%     Draws a unit cell representing grain size, location and orientation by
%     cubes or hexagonal prismes based on the their Rodrigues vectors.
%
%     INPUT:
%       grains      = <cell array>   grain structure from indexing
%
%     OPTIONAL INPUT (varargin):
%       ids         = <double>       list of grain IDs of interest
%       cmap        = <double>       can be a component of the strain tensor
%                                    {grain.strain.strainT(3,3)} or a colour map
%       hlight      = <double>       grain id-s to be highlighted {}
%       linecolor   = <string>
%       strain      = <double>       scale factor for the strain values, if wanted {0}
%                                    zero to use the unstrained unit cell
%       alpha       = <double>
%       pxsize      = <double>
%       ratio       = <double>
%       phaseid     = <int>          phase number {1}
%       type        = <string>       unit cell type {'cubic'}/'hexagonal'
%       label       = <string>
%
%       hf          =                figure handle {}
%       zoom        = <double>       zoom in of this quantity {1.2}
%       view        = <double>       3D angular view {[0 90]} (1x2)
%       legend      =
%       legendpos   =
%       multiaxes   =
%
%     OUTPUT:
%       f_handle    =                handle of the figure
%       p_handle    =                handle of the patches
%
%
%     Version 005 11-03-2013 by LNervo
%       Use cell arrays to plot multiple datasets
%
%     Version 004 15-11-2012 by LNervo
%       Use gtDrawSampleGeometry(labgeo) to create the graphics with axis and reference
%       system
%
%     Version 003 23-10-2012 by LNervo
%        Add varargin

% modify parameters/global settings
app.grainids    = [];
app.hf          = [];
app.cmap        = [];
app.latticepar  = [];
app.labgeo      = [];
app.phaseid     = 1;
app.type        = [];
app.hlight      = [];
app.scale       = [];
app.view        = [0 90];
app.showbar     = false;
app.colormap    = 'cool';
app.strain      = 0;
app.strainT     = [3 3];
app.zoom        = 1.2;

app = parse_pv_pairs(app, varargin);

app.linecolor = app.linecolor(1:numel(grains));

parameters = [];
load('parameters.mat');
labgeo     = parameters.labgeo;
latticepar = parameters.cryst.latticepar;

if isempty(app.type{1})
    app.type{1} = parameters.cryst.crystal_system;
end
if isempty(app.ratio{1})
    app.ratio{1}   = latticepar(3)/latticepar(1);
end
if isempty(app.pxsize{1})
    app.pxsize{1}  = (labgeo.pixelsizeu + labgeo.pixelsizev)/2;
end
if isempty(app.hf)
    hf = gtDrawSampleGeometry(labgeo.samenvrad, labgeo.samenvbot, labgeo.samenvtop);
else
    hf = app.hf;
end

set(hf, 'Visible', 'off');

hp = [];

for ii=1:numel(grains)
    hold(gca,'on');

    grain = grains{ii};


    if ii>1
        if numel(app.type)==1
            app.type{ii} = app.type{1};
        end
        if numel(app.pxsize)==1
            app.pxsize{ii} = app.pxsize{1};
        end
        if numel(app.ratio)==1
            app.ratio{ii} = app.ratio{1};
        end
        if numel(app.cmap)==1
            app.cmap{ii}  = app.cmap{1};
        end
        if numel(app.hlight)==1
            app.hlight{ii}  = app.hlight{1};
        end
        if numel(app.alpha)==1
            app.alpha{ii} = app.alpha{1};
        end
    end

    if isempty(app.ids{ii})
        app.ids{ii}   = 1:length(grain);
    end
    cmap = app.cmap{ii};

    if isempty(cmap)
        cmap = zeros(length(grain),1);
    else
        if size(cmap,1)==1 && size(cmap,2)>1
            cmap = repmat(cmap,length(grain),1);
        end
        % remove bkg color
        if all(cmap(1,:) == [0 0 0])
            cmap = cmap(2:end,:);
        end
    end

    grain = grain(app.ids{ii});
    cmap  = cmap(app.ids{ii},:);
    ids = app.ids{ii};

    % vertices and faces calculation
    if strcmpi(app.type{ii}, 'cubic')
        data{ii} = gtCubicUnitCell('ratio',app.ratio{ii},'draw',false);
    elseif strcmpi(app.type{ii}, 'hexagonal')
        data{ii} = gtHexagonalUnitCell('ratio',app.ratio{ii},'draw',false);
    end

    % draw unit cells
    p = [];
    for jj = 1:length(grain)

        if isfield(grain{jj}, 'centre')
            center = grain{jj}.centre;
        else
            center = grain{jj}.center;
        end
        rvec = grain{jj}.R_vector;

        if isfield(grain{jj}, 'stat') && isfield(grain{jj}.stat, 'bbxsmean') && isfield(grain{jj}.stat, 'bbysmean')
            grainsize = 0.5*app.pxsize{ii}*(grain{jj}.stat.bbxsmean + grain{jj}.stat.bbysmean)/2;
        else
            grainsize = 10;
        end

        if ~isfield(grain{jj}, 'strain')
            strainT = NaN(3);
        else
            strainT = grain{jj}.strain.strainT;
        end

        if ismember(jj, app.hlight{ii})
            hl = true;
        else
            hl = false;
        end

        p{jj} = gtPlotGrainUnitCell(center, rvec, strainT, ...
                                    data{ii}.vertices, data{ii}.faces_sides, ...
                                    hl, grainsize, cmap(jj,:), app.linecolor{ii}, ...
                                    app.strain{ii}, app.alpha{ii});
        set(p{jj}, 'UserData', [ii ids(jj)]);
        set(p{jj}, 'Tag', sprintf('data_%02d_grain_%04d',ii,grain{jj}.id));
        p{jj} = gtPlotGrainUnitCell(center, rvec, strainT, ...
                                    data{ii}.vertices, data{ii}.faces_end, ...
                                    hl, grainsize, cmap(jj,:), app.linecolor{ii}, ...
                                    app.strain{ii}, app.alpha{ii});


    end % end for jj

    hp{ii} = p;
end % end for ii

app.label = app.type;

axis equal;
zoom('out');
zoom(app.zoom)
zoom('off')
view(app.view);

if app.legend
    leg = legend('Parent',gcf,'String',app.label,'Location',app.legendpos);
end

set(gcf, 'Visible', 'on');
f_handle = gcf;

if nargout == 2
    p_handle = hp;
end

end % of main function


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUB-FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sfPlotGrainCell(grain, vertices, faces, hl, pixelsize, cmap, scale, strainscale)

if isempty(scale)
    ga = 0.5*pixelsize*(grain.stat.bbxsmean + grain.stat.bbysmean)/2;
else
    ga = 0.5*pixelsize*(grain.stat.bbxsmean + grain.stat.bbysmean)/2*scale;
end

if isfield(grain,'centre')
    gc = grain.centre;
else
  	gc = grain.center;
end

% Get or recompute orientation matrix g
if isfield(grain, 'g')
    g_grain = grain.g;
else
    g_grain = gtMathsRod2OriMat(grain.R_vector);
end

% Express vertices in cartesian SAMPLE CS
grain_vertices = ga * gtVectorCryst2Lab(vertices, g_grain);
grain_vertices_st = repmat(gc,size(vertices,1),1) + (eye(3)*grain_vertices')';

if isfield(grain,'strain') && ~any(isnan(grain.strain.strainT(:))) && strainscale ~= 0
	  grain_vertices_st = grain_vertices_st + (grain.strain.strainT*strainscale*grain_vertices')';
end

facecolor = repmat(cmap,size(faces,1),1);

if hl
    patch('Vertices', grain_vertices_st, 'Faces', faces , ...
          'FaceVertexCData', facecolor, 'FaceColor', 'flat','EdgeColor','r','LineWidth',2);
else
    patch('Vertices', grain_vertices_st, 'Faces', faces , ...
          'FaceVertexCData', facecolor, 'FaceColor', 'flat');
end

end  % of sfPlotGrainCell
