function hf = gtMakePoleFigure(varargin)
% GTMAKEPOLEFIGURE  Makes a pole figure given a list of poles
%     hf = gtMakePoleFigure(varargin)
%     -------------------------------
%
%     the poles should cover a larger angular area than required, to avoid
%     depleted pole density at the edges.  Any un-needed poles will be crops to
%     improve speed.
%
%     OPTIONAL INPUT (parse by pairs):
%       'phaseid'     = <int>        phase number {1}
%       'poles'       = <double>     poles to plot
%       'densityXYZ'  = <cell>       x,y,z values from previous polefigure calculation
%       'weights'     = <double>     can supply a vector the same length as poles to 
%                                    weight the poles
%       'range'       = <string>     'sst'/'hex'/'phi90'/{'phi360'}
%                                    standard stereographic triangle, quarter or whole
%                                    hemisphere or hexagonal sst
%       'label_poles' = <string>     'all'/'bcc'/'main'/{'none'}
%                                    all hkl poles or just the corners of the sst.
%       'mark_poles'  = <string>     'none'/'main'/'bcc'/{'all'}
%       'N'           = <int>        data point density in pole figure {100}
%       'searchR'     = <double>     smoothing radius in pole figure {0.1 rad}
%       'plot_figure' = <logical>    can choose not to plot figure {true}
%       'surf'        = <logical>    surface plot option {false}
%       'scatter'     = <logical>    scatter plot option {false}
%       'rings'       = <logical>    plot the psi rings {false}
%       'cmap'        = <double>     cmap to color grainds/poles {[]}
%       'grainids'    = <double>     grainIDs to display only some poles {[]}
%       'colormap'    =              colormap for the density plot {jet(app.N)}
%       'colorbar'    = <logical>    display the colorbar {true}
%       'hf'          =              figure handle (if existing)
%       'logscale'    = <logical>    plot log scale for axis-Z {false}
%       'crystDir'    = <double>     crystallographic direction plotted in the
%                                    pole figure used in the title and figure name {[0 0 0 1]}
%       'title'       = <string>     figure title ''
%       'export'      = <logical>    flag to export the figure or not {false}
%
%     OUTPUT:
%       hf            =              figure handle
%
%
%     Version 003 17-01-2013 by LNervo
%       Improved speed, formatted in a clearer way, splitted with other
%       functions
%
%     Version 002 26-06-2012 by LNervo
%       Add phaseid as input argument for multiple phases materials
%
%     Version 001 09-2009 by AKing
%       could be improved to pass in a list of which hkl poles to plot and to
%       label.


% read in the specs
app.phaseid     = 1;
app.poles       = [];
app.densityXYZ  = [];
app.weights     = [];
app.range       = 'phi360'; % phi90, hex, sst
app.label_poles = 'none';
app.mark_poles  = 'none';
app.N           = 100;
app.searchR     = 0.1;
app.plot_figure = true;     % true/false
app.surf        = false;    % true/false
app.scatter     = false;    % true/false
app.rings       = true;     % true/false
app.cmap        = []; % for scatter plots
app.grainids    = []; % for scatter plots
app.colormap    = jet;
app.colorbar    = true;
app.hf          = [];
app.logscale    = false;    % true/false
app.crystDir    = [0 0 0 1];
app.title       = '';
app.export      = false;
app.maxring     = 90;

app = parse_pv_pairs(app, varargin);

% figure name/title
figname = [gtGetLastDirName(pwd) '_polefigure_' strrep(num2str(app.crystDir),' ','') '__' app.range];  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% analyse the pole data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% before building figure, take only those poles that fall in the range 0-phimax,
% 0-psimax (allowing for search radius)
if isempty(app.densityXYZ)
    [poles, weights, phimax, psimax] = gtFindPolesInRange(app.poles, app.weights, app.range, []);
    if ~app.scatter
        [x, y, z] = gtCalculateDensityPhiPsi(poles, weights, [0 phimax], [0 psimax], app.searchR, app.N);
        [phi,rho,z] = cart2pol(x,y,z);
        psi = 2*atan(rho);
    else
        phi=atan2(poles(:,2),poles(:,1));
        psi=acos(poles(:,3));
        rho=tan(psi/2);
        [x,y]=pol2cart(phi,rho);
        z=[];
        app.colorbar = false;
    end
else
    x = app.densityXYZ{1};
    y = app.densityXYZ{2};
    z = app.densityXYZ{3};
    [phi,rho,z] = cart2pol(x,y,z);
    psi = 2*atan(rho);
end

if app.logscale && ~isempty(z)
    z = log10(z);
    disp('Using log scale for axis Z')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build the figure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if app.plot_figure
    
    if ~isempty(app.hf)
        hf = app.hf;
    else
        hf = figure();
    end
    hold(gca,'on');
    % plot the poles
    if app.surf
        surf(x,y,z)
        shading('interp')
        disp('Using colour interpolation for the surface plot')
        figname = [figname '_surf'];
    elseif app.scatter
        if ~isempty(app.cmap)
          
            figname = [figname '_cmap'];
            if all(app.cmap(1,:) == [0 0 0])
                app.cmap = app.cmap(2:end,:);
            end
            if length(app.cmap) ~= length(x)
                tmp = app.cmap;
                ids = app.poles(:,1);
                
            end
            if ~isempty(app.grainids)
              app.cmap = app.cmap(app.grainids,:);
              x = x(app.grainids);
              y = y(app.grainids);
            end
            scatter(gca, x,y,10,app.cmap,'filled')
        else
            scatter(gca, x,y,10,'ob','filled')
        end
        disp('Scatter plot with markersize 10 is used')
        figname = [figname '_scatter'];
    else
        contourf(x,y,z,round(app.N/10));
        disp(['Using ' num2str(round(app.N/10)) ' levels for the contour plot'])
        figname = [figname '_contour'];
    end
    
    hold(gca,'on')

    if ~app.scatter
        label_z=(max(z(:))+1);
    else
        label_z=1;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % mark poles
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~strcmp(app.mark_poles, 'none')
        
        % finds poles in the psi/phi range to be plotted
        [uvw_poles,~,~,~] = gtFindPolesInRange([], [], app.range, app.mark_poles);        
        % converts poles from cartesian to spherical coordinates
        [phi_uvw,psi_uvw,r_uvw]=cart2sph(uvw_poles(:,1),uvw_poles(:,2),uvw_poles(:,3));
        % bring phi in [0,2pi] range from [-pi,+pi] range
        phi_uvw=phi_uvw+pi;
        % bring psi in [0,pi] range from [-pi/2,pi/2] range
        psi_uvw=pi/2-psi_uvw;
        % project poles on the plane
        rho_uvw=r_uvw.*tan(psi_uvw/2);
        % go from polar to cartesian space
        [x_uvw,y_uvw]=pol2cart(phi_uvw,rho_uvw);

        % drawing the poles
        plot3(x_uvw, y_uvw, ones(size(x_uvw))*label_z, 'k*')
        
    end % end mark_poles
    
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % psi rings
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if app.rings
        for ring=15:15:app.maxring
            ring_phi=deg2rad([0:3:360 0]);
            ring_psi=deg2rad(ring)*ones(size(ring_phi));
            ring_r=tan(ring_psi/2);
            ring_z=repmat(label_z,size(ring_phi));
            [ring_x,ring_y]=pol2cart(ring_phi,ring_r);
            plot3(ring_x, ring_y, ring_z, 'w-');
            text(ring_x(1)+0.01, 0, label_z, num2str(ring), 'Color', [1 1 1]);
        end
    end
       
    if strcmp(app.range, 'sst')
        %can be replaced by gtIPFCmapCubicKey to get the figure without the patch
        
        %line from 101 to 111 is z=x
        line_phi=[0:0.01:pi/4 pi/4];
        linex=cos(line_phi);
        linez=linex;
        liney=sin(line_phi);
        line_points=[linex' liney' linez'];
        %normalise
        line_points=line_points./repmat(sqrt(sum(line_points.*line_points, 2)), 1, 3);
        %convert to stereographic proj.
        line_psi=acos(line_points(:,3));
        line_r=tan(line_psi/2);
        [line_x,line_y]=pol2cart(line_phi', line_r);
        
        %white patch over unwanted bit of figure
        mask_x=[line_x ; sqrt(2)-1+0.045 ; sqrt(2)-1+0.045 ; line_x(1)];
        mask_y=[line_y ; line_y(end) ; 0 ; 0];
        mask_z=ones(size(mask_y))*(max(z(:))+0.5); %between the pole points and the surface
        p=patch(mask_x, mask_y, mask_z, 'w');
        set(p, 'edgecolor', 'w')
        
        %black outline on polefigure
        mask_x=[line_x ; 0 ; line_x(1)];
        mask_y=[line_y ; 0 ; 0];
        mask_z=ones(size(mask_y))*(max(z(:))+1);
        plot3(mask_x, mask_y, mask_z, 'k')
        
        %tweak axis limits and therefore adjust camera position
        set(gca, 'xlim', [-0.02 sqrt(2)-1+0.045])
        set(gca, 'ylim', [-0.025 0.3769])
        %set(gca, 'CameraPosition', [(sqrt(2)-1+0.025)/2 (-0.025+0.3769)/2 (max(z(:))+2)]);
        set(gca, 'CameraUpVector', [0 1 0]);
        
    end % end range
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % label poles
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~strcmp(app.label_poles, 'none')
        if strcmp(app.label_poles, 'all') || strcmp(app.label_poles, 'main') || strcmp(app.label_poles, 'bcc')
            %label the poles son the corners of the sst
            text(0, 0-0.015, label_z, '(001)')
            text(0.366+0.01, 0.366-0.005, label_z, '(111)')
            text(0.4142-0.015, -0.015, label_z, '(101)')
        end
        
        if strcmp(app.label_poles, 'all')
            %label the long list of poles
            text(0.1213-0.04, 0.1213, label_z, '(114)')
            text(0.1583-0.04, 0.1583, label_z, '(113)')
            text(0.2247-0.04, 0.2247, label_z, '(112)')
            text(0.2808-0.04, 0.2808, label_z, '(223)')
            text(0.3052-0.045, 0.3052, label_z, '(334)')
            
            text(0.3845+0.01, 0.2884, label_z, '(434)')
            text(0.3901+0.01, 0.2601, label_z, '(323)')
            text(0.4000+0.01, 0.2000, label_z, '(212)')
            text(0.4077+0.01, 0.1359, label_z, '(313)')
            text(0.4105+0.01, 0.1026, label_z, '(414)')
            
            text(0.1231-0.02, -0.015, label_z, '(104)')
            text(0.1623-0.01, -0.015, label_z, '(103)')
            text(0.2361-0.015, -0.015, label_z, '(102)')
            text(0.3028-0.02, -0.015, label_z, '(203)')
            text(0.3333-0.01, -0.015, label_z, '(304)')
            
            text(0.2967+0.01, 0.1483, label_z, '(213)')
            text(0.2330+0.01, 0.1165, label_z, '(214)')
            text(0.3297+0.01, 0.1099, label_z, '(314)')
            text(0.3197+0.01, 0.2131, label_z, '(324)')
        elseif strcmp(app.label_poles, 'bcc')
            %label bcc slip planes
            text(0.2247-0.04, 0.2247, label_z, '(112)')
            text(0.3, 0.153, label_z, '(213)')
        end
    end % end label_poles
    
    % sample reference
    text(1+0.1,0,    'Xs','Color',[0 0 0]);
    text(-0.05,1+0.1,'Ys','Color',[0 0 0]);
    text(-0.05,0,    'Zs','Color',[1 1 1]);
    
    if app.colorbar
        colorbar('Location', 'manual','Position',[0.9351 0.1089 0.0424 0.7840],'YAxisLocation','left');
    end
    if ~strcmp(app.colormap, 'IPF')
        colormap(app.colormap)
%     elseif app.scatter
%         hf2=gtIPFCmapHexagonalKey();
%         ha=get(hf2,'CurrentAxes');
%         set(ha,'Position',[0.75 0.1 0.2 0.2]);
%         c=copyobj(ha,gcf);
%         figname = [figname 'IPF'];
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % tidy the figure
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    set(gca, 'XLim', [-1.2 1.2]);
    set(gca, 'YLim', [-1.2 1.2]);
    set(gca,'DataAspectRatio',[.2 .2 1]);
    set(gca,'Position',[-0.0276 0.0079 0.9122 0.934]);
    
    set(gca,'View',[0 90]);
    set(gca,'Visible','off');
    
    if ~isempty(app.title)
        set(get(gca,'Title'),'String',app.title);
        set(get(gca,'Title'),'Visible','on')
    end
    set(gcf,'Position',[1259 470 589 537]);
    set(gcf,'Color',[1 1 1]);
    set(gcf,'UserData',{x,y,z;phi,psi,rho});
    set(gcf,'Visible','on');
    hf = gcf;
    
    if app.export
        export(figname,hf);
    else
        disp('Figure name with description to export it:')
        disp(figname)
        assignin('base','figname',figname);
        disp('the variable figname has been assigned to the workspace')
        % set title of the figure using figname
        disp('To change the title do:')
        disp('    gtPlotSettings([],[],strrep(figname,''_'','' ''))');
    end
else
    hf.x = x;
    hf.y = y;
    hf.z = z;
    disp('hf contains x,y,z values of the pole figure')
end % end plot_figure

end % end of function
