function list = gtCrystCalculateReflections(latticepar, mintheta, maxtheta, sg, filename)
% GTCRYSTCALCULATEREFLECTIONS  Calculate the reflections list using python 
%                              from xfab library in fable
%     list = gtCrystCalculateReflections(latticepar, mintheta, maxtheta, sg, filename)
%     --------------------------------------------------------------------------------
%
%     INPUT: 
%       latticepar      = lattice parameters <double 1x6>
%       mintheta        = minimum sin(theta)/lambda allowed <double> {0.0001}
%       maxtheta        = maximum sin(theta)/lambda allowed <double>
%       sg              = spacegroup number <int>
%       filename        = filename for reflections list <string> {'reflections_*.txt'}
%
%     OUTPUT:
%       list = structure with fields (the same with 'sp' for the full lists)
%           .hkl       = reflection list <double>
%           .hkil      = hkil indexes if needed <double>
%           .theta     = theta value for each reflection <double>
%           .thetatype = theta type <double>
%           .sintheta  = sind(theta)/lambda <double>
%           .dspacing  = d-spacing <double>
%           .mult      = multiplicity of each reflections <double>
%
%
%     Version 002 12-12-2012 by LNervo
%       Compact output as a structure
%
%     Version 001 03-12-2012 by LNervo


if ~exist('latticepar','var') || isempty(latticepar)
    parameters = [];
    load parameters;
    latticepar = parameters.cryst(1).latticepar;
end
if ~exist('mintheta','var') || isempty(mintheta)
    mintheta = 0.0001;
end
if ~exist('maxtheta','var') || isempty(maxtheta)
    load parameters;
    tmp = parameters.labgeo.detanglemax;
    maxtheta = sind(tmp)/gtConvEnergyToWavelength(parameters.acq.energy);
    clear tmp;
end
if ~exist('sg','var') || isempty(sg)
    load parameters;
    sg = parameters.cryst(1).spacegroup;
end
if ~exist('filename','var') || isempty(filename)
    filename = 'reflections_';
end

parameters = [];
load('parameters.mat');
energy = parameters.acq.energy;

latticepar_str = sprintf('%f ',latticepar);
latticepar_str = strtrim(latticepar_st);
% python script to be used
global GT_MATLAB_HOME;
script_file = fullfile(GT_MATLAB_HOME, 'zUtil_Python', 'reflections_list.py');

% run the command for all the reflections
cmd = sprintf('%s %s %f %f %d %s %s', ...
              script_file,latticepar_str,mintheta,maxtheta,sg,'all',filename);
[~, msg] = gtPythonCommand(cmd, true);
disp(msg);

% run the command for only the reflections unique
cmd = sprintf('%s %s %f %f %d %s %s', ...
              script_file,latticepar_str,mintheta,maxtheta,sg,'unique',filename);
[~, msg] = gtPythonCommand(cmd, true);
disp(msg);

% read the full list of reflections
[~,Cmat] = gtReadTextFile([filename 'all.txt'],'%f %f %f %f','\n','#',[1 3]);

% extracting information
hklsp = Cmat(:,1:3);
sinthetasp = Cmat(:,4);
% getting 4-indexes notation
hkilsp = hklsp;
hkilsp(:,4) = hkilsp(:,3);
hkilsp(:,3) = -hkilsp(:,1)-hkilsp(:,2);

% extracting theta
thetasp = asind(sinthetasp)*gtConvEnergyToWavelength(energy);
% calculation of d-spacing
Bmat = gtCrystHKL2CartesianMatrix(latticepar);
dspacingsp = gtCrystDSpacing(hklsp',Bmat)';
% calculation of the multiplicity
[sintheta,ind,thetatypesp] = unique(sinthetasp,'stable');
mult = [ind(2:end)',length(thetatypesp)+1]-ind';

hkl       = hklsp(ind,:);
theta     = thetasp(ind);
hkil      = hkilsp(ind,:);
thetatype = thetatypesp(ind);
dspacing  = dspacingsp(ind);

disp('  h   k   i   l     theta   thetatype  mult    d-spacing');
disp('----------------------------------------------------------');
for ii=1:size(ind,1)
    fprintf( '%3d %3d %3d %3d %9.3f %5d %10d %10.3f\n',hkilsp(ind(ii),:),thetasp(ind(ii)),thetatypesp(ind(ii)),mult(ii),dspacingsp(ind(ii)) );
end

% creating the output
list.latticepar  = latticepar;
list.spacegroup  = sg;
list.energy      = energy;
list.hkl         = hkl';
list.hklsp       = hklsp';
list.hkil        = hkil';
list.hkilsp      = hkilsp';
list.theta       = theta';
list.thetasp     = thetasp';
list.thetatype   = thetatype';
list.thetatypesp = thetatypesp';
list.sintheta    = sintheta';
list.sinthetasp  = sinthetasp';
list.dspacing    = dspacing';
list.dspacingsp  = dspacingsp';
list.mult        = mult;


end % end of function
