/*
 * internal_cell_defs.h
 *
 *  Created on: Jul 11, 2012
 *      Author: vigano
 */

#ifndef INTERNAL_CELL_DEFS_H_
#define INTERNAL_CELL_DEFS_H_

#include <math.h>
#include <omp.h>
#include "mex.h"

# define ROUND_DOWN(x, s) ((x) & ~((s)-1))

#if defined(__SSE2__) || defined(__SSE2_MATH__)
# include <xmmintrin.h>
#endif

mxClassID
getMxClassOfCell(const mxArray * cell)
{
  const mwSize numElems = mxGetNumberOfElements(cell);
  if (numElems > 0) {
    const mxArray * const firstArray = mxGetCell(cell, 0);
    const mxClassID firstClass = mxGetClassID(firstArray);
    if (firstClass == mxSINGLE_CLASS || firstClass == mxDOUBLE_CLASS) {
      for(mwIndex count = 1; count < numElems; count++) {
        const mxArray * const array = mxGetCell(cell, count);
        if (firstClass != mxGetClassID(array)) {
          // Damn mixed types!
          return mxUNKNOWN_CLASS;
        }
      }
      // OK it is a float type!
      return firstClass;
    } else {
      // It is still an error!
      return firstClass;
    }
  } else {
    return mxUNKNOWN_CLASS;
  }
}

template<typename Type1, typename Type2>
class inner_assign {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData2);
  }
};

template<typename Type1, typename Type2>
class inner_pow {
  const double exponent;
public:
  inner_pow(const double & _exp) : exponent(_exp) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return pow(inData2, exponent);
  }
};

template<typename Type1, typename Type2>
class inner_sqrt {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return sqrt(inData2); // __builtin_ia32_sqrtpd
  }
};

#if defined(__SSE2__) || defined(__SSE2_MATH__)
template<>
class inner_sqrt<const __v2df, const __v2df> {
public:
  const __v2df
  operator()(const __v2df & inData1, const __v2df & inData2) const throw()
  {
    return _mm_sqrt_pd(inData2);
  }
};

template<>
class inner_sqrt<const __v4sf, const __v4sf> {
public:
  const __v4sf
  operator()(const __v4sf & inData1, const __v4sf & inData2) const throw()
  {
    return _mm_sqrt_ps(inData2);
  }
};
#endif

template<typename Type1, typename Type2>
class inner_sum {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 + inData2);
  }
};

template<typename Type1, typename Type2>
class inner_prod {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 * inData2);
  }
};

template<typename Type1, typename Type2>
class inner_sub {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 - inData2);
  }
};

template<typename Type1, typename Type2>
class inner_div {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 / inData2);
  }
};


template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sum_scale {
  const TypeScalar scale;
public:
  inner_sum_scale(const TypeScalar & _scale) : scale(_scale) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 + scale * inData2);
  }
};

template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sum_FISTA_scale {
  const TypeScalar scale;
public:
  inner_sum_FISTA_scale(const TypeScalar & _scale) : scale(_scale) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 + scale * (inData1 - inData2));
  }
};

template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sub_scale {
  const TypeScalar scale;
public:
  inner_sub_scale(const TypeScalar & _scale) : scale(_scale) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 - inData2 * scale);
  }
};


template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sum_scalar {
  const TypeScalar scalar;
public:
  inner_sum_scalar(const TypeScalar & _scalar) : scalar(_scalar) { }

  const Type1
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData2 + scalar);
  }
};

template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_prod_scalar {
  const TypeScalar scalar;
public:
  inner_prod_scalar(const TypeScalar & _scalar) : scalar(_scalar) { }

  const Type1
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData2 * scalar);
  }
};

template<mxClassID type>
inline void
cell_allocate(mxArray * & outArray, const mxArray * const inArray)
{
  const mwSize numCells = mxGetNumberOfElements(inArray);
  const mwSize numCellDims = mxGetNumberOfDimensions(inArray);
  const mwSize * cellDims = mxGetDimensions(inArray);
  outArray = mxCreateCellArray(numCellDims, cellDims);

  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * inCell = mxGetCell(inArray, cellIdx);
    const mwSize numDims = mxGetNumberOfDimensions(inCell);
    const mwSize * dims = mxGetDimensions(inCell);
    mxArray * const outCell = mxCreateNumericArray(numDims, dims, type, mxREAL);
    mxSetCell(outArray, cellIdx, outCell);
  }
}

#if defined(__SSE2__) || defined(__SSE2_MATH__)
template<typename Type, const bool allocate, typename Function, typename Function2>
inline void
cell_iteration_sse(mxArray * & outArray, const mxArray * const inArray,
    const Function & func, const Function2 & func2)
{
  if (allocate) {
    if (sizeof(Type) == 8) {
      cell_allocate<mxDOUBLE_CLASS>(outArray, inArray);
    } else {
      cell_allocate<mxSINGLE_CLASS>(outArray, inArray);
    }
  }

  const mwSize numCells = mxGetNumberOfElements(inArray);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell = mxGetCell(inArray, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell);
      const Type * const inData = (const Type *) mxGetData(inCell);
      Type * const outData = (Type *) mxGetData(outCell);

      cell_inner_cycle_sse(outData, outData, inData, numElems, func, func2);
    }
  }
}

template<typename Type, typename Function, typename Function2>
inline void
cell_iteration_copy_sse(mxArray * & outArray, const mxArray * const inArray1,
    const mxArray * const inArray2,
    const Function & func, const Function2 & func2)
{
  if (sizeof(Type) == 8) {
    cell_allocate<mxDOUBLE_CLASS>(outArray, inArray1);
  } else {
    cell_allocate<mxSINGLE_CLASS>(outArray, inArray1);
  }

  const mwSize numCells = mxGetNumberOfElements(inArray1);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
      const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell1);
      const Type * const inData1 = (const Type *) mxGetData(inCell1);
      const Type * const inData2 = (const Type *) mxGetData(inCell2);
      Type * const outData = (Type *) mxGetData(outCell);

      cell_inner_cycle_sse(outData, inData1, inData2, numElems, func, func2);
    }
  }
}

template<typename Type, typename FunctionVect, typename FunctionNonVect>
inline void
cell_inner_cycle_sse(Type * const __restrict outData,
    const Type * const __restrict inData1,
    const Type * const __restrict inData2,
    const mwSize & numElems, FunctionVect & func, FunctionNonVect & func2)
{
  typedef Type vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const mwSize unrolling = 8;
  const mwSize shift = 16 / sizeof(Type);
  const mwSize block = shift * unrolling;
  const mwSize numRoundElems = ROUND_DOWN(numElems, block);
#pragma omp for nowait
  for(mwIndex elemIdx = 0; elemIdx < numRoundElems; elemIdx += block) {
    const vVvf inV11 = *((const vVvf *)&inData1[elemIdx + 0 * shift]);
    const vVvf inV12 = *((const vVvf *)&inData1[elemIdx + 1 * shift]);
    const vVvf inV13 = *((const vVvf *)&inData1[elemIdx + 2 * shift]);
    const vVvf inV14 = *((const vVvf *)&inData1[elemIdx + 3 * shift]);

    const vVvf inV21 = *((const vVvf *)&inData2[elemIdx + 0 * shift]);
    const vVvf inV22 = *((const vVvf *)&inData2[elemIdx + 1 * shift]);
    const vVvf inV23 = *((const vVvf *)&inData2[elemIdx + 2 * shift]);
    const vVvf inV24 = *((const vVvf *)&inData2[elemIdx + 3 * shift]);

    *((vVvf *)&outData[elemIdx + 0 * shift]) = func(inV11, inV21);
    *((vVvf *)&outData[elemIdx + 1 * shift]) = func(inV12, inV22);
    *((vVvf *)&outData[elemIdx + 2 * shift]) = func(inV13, inV23);
    *((vVvf *)&outData[elemIdx + 3 * shift]) = func(inV14, inV24);

    const vVvf inV31 = *((const vVvf *)&inData1[elemIdx + 4 * shift]);
    const vVvf inV32 = *((const vVvf *)&inData1[elemIdx + 5 * shift]);
    const vVvf inV33 = *((const vVvf *)&inData1[elemIdx + 6 * shift]);
    const vVvf inV34 = *((const vVvf *)&inData1[elemIdx + 7 * shift]);

    const vVvf inV41 = *((const vVvf *)&inData2[elemIdx + 4 * shift]);
    const vVvf inV42 = *((const vVvf *)&inData2[elemIdx + 5 * shift]);
    const vVvf inV43 = *((const vVvf *)&inData2[elemIdx + 6 * shift]);
    const vVvf inV44 = *((const vVvf *)&inData2[elemIdx + 7 * shift]);

    *((vVvf *)&outData[elemIdx + 4 * shift]) = func(inV31, inV41);
    *((vVvf *)&outData[elemIdx + 5 * shift]) = func(inV32, inV42);
    *((vVvf *)&outData[elemIdx + 6 * shift]) = func(inV33, inV43);
    *((vVvf *)&outData[elemIdx + 7 * shift]) = func(inV34, inV44);
  }
#pragma omp for nowait
  for(mwIndex elemIdx = numRoundElems; elemIdx < numElems; elemIdx++) {
    outData[elemIdx] = func2(inData1[elemIdx], inData2[elemIdx]);
  }
}

template<typename Type>
inline Type
cell_square_norm_sse(const mxArray * const inArray)
{
  typedef Type vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const mwSize unrolling = 16;
  const mwSize shift = 16 / sizeof(Type);
  const mwSize block = shift * unrolling;
  Type temp_sum = 0;
  vVvf accumul = (sizeof(Type) == 8) ? (vVvf) _mm_set1_pd(0.0) : (vVvf) _mm_set1_ps(0.0);

  const mwSize numCells = mxGetNumberOfElements(inArray);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell = mxGetCell(inArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell);
      const Type * const inData = (const Type *) mxGetData(inCell);

      const mwSize numRoundElems = ROUND_DOWN(numElems, block);

#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = 0; count < numRoundElems; count += block) {
        const vVvf in00 = *((vVvf *)&inData[count +  0 * shift]);
        const vVvf in02 = *((vVvf *)&inData[count +  1 * shift]);
        const vVvf in04 = *((vVvf *)&inData[count +  2 * shift]);
        const vVvf in06 = *((vVvf *)&inData[count +  3 * shift]);
        const vVvf in08 = *((vVvf *)&inData[count +  4 * shift]);
        const vVvf in10 = *((vVvf *)&inData[count +  5 * shift]);
        const vVvf in12 = *((vVvf *)&inData[count +  6 * shift]);
        const vVvf in14 = *((vVvf *)&inData[count +  7 * shift]);
        const vVvf accumulInternal1 = ( in00 * in00 + in02 * in02
                                      + in04 * in04 + in06 * in06
                                      + in08 * in08 + in10 * in10
                                      + in12 * in12 + in14 * in14 );

        const vVvf in16 = *((vVvf *)&inData[count +  8 * shift]);
        const vVvf in18 = *((vVvf *)&inData[count +  9 * shift]);
        const vVvf in20 = *((vVvf *)&inData[count + 10 * shift]);
        const vVvf in22 = *((vVvf *)&inData[count + 11 * shift]);
        const vVvf in24 = *((vVvf *)&inData[count + 12 * shift]);
        const vVvf in26 = *((vVvf *)&inData[count + 13 * shift]);
        const vVvf in28 = *((vVvf *)&inData[count + 14 * shift]);
        const vVvf in30 = *((vVvf *)&inData[count + 15 * shift]);
        const vVvf accumulInternal2 = ( in16 * in16 + in18 * in18
                                      + in20 * in20 + in22 * in22
                                      + in24 * in24 + in26 * in26
                                      + in28 * in28 + in30 * in30 );
        accumul += accumulInternal1 + accumulInternal2;
      }
#pragma omp for reduction(+:temp_sum) nowait
      for(mwIndex count = numRoundElems; count < numElems; count++) {
        temp_sum += inData[count] * inData[count];
      }
    }
  }
  const Type * const __restrict accumuld = (Type *)&accumul;
  if (shift == 2) {
    return (temp_sum + accumuld[0] + accumuld[1]);
  } else {
    return (temp_sum + accumuld[0] + accumuld[1] + accumuld[2] + accumuld[3]);
  }
}

template<typename Type>
inline Type
cell_dot_product_sse(const mxArray * const inArray1, const mxArray * const inArray2)
{
  typedef Type vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const mwSize unrolling = 16;
  const mwSize shift = 16 / sizeof(Type);
  const mwSize block = shift * unrolling;
  Type temp_sum = 0;
  vVvf accumul = (sizeof(Type) == 8) ? (vVvf) _mm_set1_pd(0.0) : (vVvf) _mm_set1_ps(0.0);

  const mwSize numCells = mxGetNumberOfElements(inArray1);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
      const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell1);
      const Type * const inData1 = (const Type *) mxGetData(inCell1);
      const Type * const inData2 = (const Type *) mxGetData(inCell2);

      const mwSize numRoundElems = ROUND_DOWN(numElems, block);

#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = 0; count < numRoundElems; count += block) {
        const vVvf in00 = *((vVvf *)&inData1[count +  0 * shift])
                        * *((vVvf *)&inData2[count +  0 * shift]);
        const vVvf in01 = *((vVvf *)&inData1[count +  1 * shift])
                        * *((vVvf *)&inData2[count +  1 * shift]);
        const vVvf in02 = *((vVvf *)&inData1[count +  2 * shift])
                        * *((vVvf *)&inData2[count +  2 * shift]);
        const vVvf in03 = *((vVvf *)&inData1[count +  3 * shift])
                        * *((vVvf *)&inData2[count +  3 * shift]);
        const vVvf in04 = *((vVvf *)&inData1[count +  4 * shift])
                        * *((vVvf *)&inData2[count +  4 * shift]);
        const vVvf in05 = *((vVvf *)&inData1[count +  5 * shift])
                        * *((vVvf *)&inData2[count +  5 * shift]);
        const vVvf in06 = *((vVvf *)&inData1[count +  6 * shift])
                        * *((vVvf *)&inData2[count +  6 * shift]);
        const vVvf in07 = *((vVvf *)&inData1[count +  7 * shift])
                        * *((vVvf *)&inData2[count +  7 * shift]);
        const vVvf accumulInternal1 = ( in00 + in01 + in02 + in03
                                      + in04 + in05 + in06 + in07 );

        const vVvf in08 = *((vVvf *)&inData1[count +  8 * shift])
                        * *((vVvf *)&inData2[count +  8 * shift]);
        const vVvf in09 = *((vVvf *)&inData1[count +  9 * shift])
                        * *((vVvf *)&inData2[count +  9 * shift]);
        const vVvf in10 = *((vVvf *)&inData1[count + 10 * shift])
                        * *((vVvf *)&inData2[count + 10 * shift]);
        const vVvf in11 = *((vVvf *)&inData1[count + 11 * shift])
                        * *((vVvf *)&inData2[count + 11 * shift]);
        const vVvf in12 = *((vVvf *)&inData1[count + 12 * shift])
                        * *((vVvf *)&inData2[count + 12 * shift]);
        const vVvf in13 = *((vVvf *)&inData1[count + 13 * shift])
                        * *((vVvf *)&inData2[count + 13 * shift]);
        const vVvf in14 = *((vVvf *)&inData1[count + 14 * shift])
                        * *((vVvf *)&inData2[count + 14 * shift]);
        const vVvf in15 = *((vVvf *)&inData1[count + 15 * shift])
                        * *((vVvf *)&inData2[count + 15 * shift]);
        const vVvf accumulInternal2 = ( in08 + in09 + in10 + in11
                                      + in12 + in13 + in14 + in15 );
        accumul += accumulInternal1 + accumulInternal2;
      }
#pragma omp for reduction(+:temp_sum) nowait
      for(mwIndex count = numRoundElems; count < numElems; count++) {
        temp_sum += inData1[count] * inData2[count];
      }
    }
  }
  const Type * const __restrict accumuld = (Type *)&accumul;
  if (shift == 2) {
    return (temp_sum + accumuld[0] + accumuld[1]);
  } else {
    return (temp_sum + accumuld[0] + accumuld[1] + accumuld[2] + accumuld[3]);
  }
}
#endif

template<typename Type, const bool allocate, typename Function>
inline void
cell_iteration(mxArray * & outArray, const mxArray * const inArray,
    const Function & func)
{
  const mwSize numCells = mxGetNumberOfElements(inArray);
  if (allocate) {
    if (sizeof(Type) == 8) {
      cell_allocate<mxDOUBLE_CLASS>(outArray, inArray);
    } else {
      cell_allocate<mxSINGLE_CLASS>(outArray, inArray);
    }
  }

#pragma omp parallel for
  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * const inCell = mxGetCell(inArray, cellIdx);
    mxArray * const outCell = mxGetCell(outArray, cellIdx);

    const mwSize numElems = mxGetNumberOfElements(inCell);
    const Type * const inData = (const Type *) mxGetData(inCell);
    Type * const outData = (Type *) mxGetData(outCell);

    for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++) {
      outData[elemIdx] = func(outData[elemIdx], inData[elemIdx]);
    }
  }
}

template<typename Type, typename Function>
inline void
cell_iteration_copy(mxArray * & outArray, const mxArray * const inArray1,
    const mxArray * const inArray2, const Function & func)
{
  const mwSize numCells = mxGetNumberOfElements(inArray1);
  if (sizeof(Type) == 8) {
    cell_allocate<mxDOUBLE_CLASS>(outArray, inArray1);
  } else {
    cell_allocate<mxSINGLE_CLASS>(outArray, inArray1);
  }

#pragma omp parallel for
  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
    const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);
    mxArray * const outCell = mxGetCell(outArray, cellIdx);

    const mwSize numElems = mxGetNumberOfElements(inCell1);
    const Type * const inData1 = (const Type *) mxGetData(inCell1);
    const Type * const inData2 = (const Type *) mxGetData(inCell2);
    Type * const outData = (Type *) mxGetData(outCell);

    for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++) {
      outData[elemIdx] = func(inData1[elemIdx], inData2[elemIdx]);
    }
  }
}


#endif /* INTERNAL_CELL_DEFS_H_ */
