classdef GtGrainConflicts < handle
% GTGRAINCONFLICTS  Class that contains all the information about conflicts of a given phase.
% All the grains are organized in a performance oriented structure
% get/set methods are provided, to aid the consistent use of the class.

    properties
        grain     = { };       % List of grains conflict info
        analyser  = { };       % List of analyser for each grain (see class GtAnalyser)
        conflicts = { };       % List of conflicts for each grain (see class GtConflict)
    end

    methods
        function obj = GtGrainConflicts(numberOfGrains)
        % GTGRAINCONFLICTS/GTGRAINCONFLICTS( numberOfGrains )
        %     Initializes the GtGrainConflicts with 'numberOfGrains'
            if (~exist('numberOfGrains', 'var'))
                numberOfGrains = 0;
            end
            obj.initGrains(numberOfGrains);
        end

        function initGrains(obj, number)
        % GTGRAINCONFLICTS/INITGRAINS( number )
        %     Initializes the GtGrainConflicts to 'number' of grains
            if number<0
                gtError('GtGrainConflicts:wrongArgument','The number of grains must be positive...Quitting')
            end
            obj.grain     = cell(number,1);
            obj.analyser  = cell(number,1);
            obj.conflicts = cell(number,1);
            
            for ii=1:number
                obj.resetGrainId(ii);
            end
        end
        
        function initConflict(obj, grainid)
        % GTGRAINCONFLICTS/INITCONFLICT( grainid )
        %     Initializes analyser and conflicts for grainid
            obj.resetAnalyser(grainid);
            obj.resetConflicts(grainid);
        end
        

        function pushGrainId(obj)
        % GTGRAINCONFLICTS/PUSHGRAINID()
        %     Adds a new empty object at the end of each property
            obj.grain{end+1}     = [];
            obj.analyser{end+1}  = GtAnalyser();
            obj.conflicts{end+1} = repmat(GtConflict(),0,1);
        end
        
        function deleteGrainId(obj, grainid)
        % GTGRAINCONFLICTS/DELETEGRAINID( grainid )
        %     Deletes objects number 'grainid' from each property
            obj.grain(grainid)     = [];
            obj.analyser(grainid)  = [];
            obj.conflicts(grainid) = [];
        end

        
        function resetGrainId(obj, grainid)
        % GTGRAINCONFLICTS/RESETGRAINID( grainid )
        %     Resets to zero all the properties for grainid
            obj.resetGrain(grainid);
            obj.resetAnalyser(grainid);
            obj.resetConflicts(grainid);
        end
        
        
        function setAnalyserVars(obj, grainid, varargin)
        % GTGRAINCONFLICTS/SETANALYSERVARS( grainid, varargin )
            tmp = obj.getAnalyser(grainid);
            
        end
        
        function setConflictsVars(obj, grainid, conflictid, varargin)
        % GTGRAINCONFLICTS/SETCONFLICTSVARS( grainid, conflictid, varargin )
            
        end
               
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Setter Functions

        % grain 
        function setGrain(obj, grainid, grain)
        % GTGRAINCONFLICTS/SETGRAIN( grainid, grain )
            obj.grain{grainid} = grain;  
        end
        
        function resetGrain(obj, grainid)
        % GTGRAINCONFLICTS/RESETGRAIN( grainid )
        %     Reset property 'grain' for grainid
            obj.grain{grainid} = [];
        end
        
        % analyser fields
        function setTotSpots(obj, grainid, tot)
        % GTGRAINCONFLICTS/SETTOTSPOTS( grainid, tot )
            obj.analyser{grainid}.totspots = tot;
        end

        function setIdSpot(obj, grainid, id)
        % GTGRAINCONFLICTS/SETIDSPOT( grainid, id )
            obj.analyser{grainid}.idspot = id;
        end

        function setIdSpotRow(obj, grainid, row, ind)
        % GTGRAINCONFLICTS/SETIDSPOTROW( grainid, row, ind )
            obj.analyser{grainid}.idspot(row,:) = ind;
        end
 
        function setTotGrains(obj, grainid, num)
        % GTGRAINCONFLICTS/SETTOTGRAINS( grainid, num )
            obj.analyser{grainid}.totgrains = num;
        end

        function setIdConflict(obj, grainid, id)
        % GTGRAINCONFLICTS/SETIDCONFLICT( grainid, id )
            obj.analyser{grainid}.idconflict = id;
        end 

        function setFlag(obj, grainid, f)
        % GTGRAINCONFLICTS/SETFLAG( grainid, f )
            obj.analyser{grainid}.flag = f;
        end

        function setDifspotID(obj, grainid, spots)
        % GTGRAINCONFLICTS/SETDIFSPOTID( grainid, spots )
            obj.analyser{grainid}.difspotID = spots;
        end
        
        function setAnalyser(obj, grainid, analyser)
        % GTGRAINCONFLICTS/SETANALYSER( grainid, analyser )
            obj.analyser{grainid} = analyser;
        end
        
        function resetAnalyser(obj, grainid)
        % GTGRAINCONFLICTS/RESETANALYSER( grainid )
        %     Reset property 'analyser' for grainid
            obj.analyser{grainid} = GtAnalyser();
        end
        
        % conflicts fields
        function setDifspots(obj, grainid, conflictid, spots)
        % GTGRAINCONFLICTS/SETDIFSPOTS( grainid, conflictid, spots )
            obj.conflicts{grainid}(conflictid).difspots = spots;
        end

        function setMaxNum(obj, grainid, conflictid, max)
        % GTGRAINCONFLICTS/SETMAXNUM( grainid, conflictid, max )
            obj.conflicts{grainid}(conflictid).maxnum = max;
        end

        function setMerge(obj, grainid, conflictid, m)
        % GTGRAINCONFLICTS/SETMERGE( grainid, conflictid, m )
            obj.conflicts{grainid}(conflictid).merge = m;
        end

        function setAllEquals(obj, grainid, conflictid, alleq)
        % GTGRAINCONFLICTS/SETALLEQUALS( grainid, conflictid, alleq )        
            obj.conflicts{grainid}(conflictid).allequals = alleq;
        end
        
        function setDelete(obj, grainid, conflictid, del)
        % GTGRAINCONFLICTS/SETDELETE( grainid, conflictid, del )
            obj.conflicts{grainid}(conflictid).delete = del;
        end
        
        function setConflictId(obj, grainid, conflictid, conflict)
        % GTGRAINCONFLICTS/SETCONFLICTID( grainid, conflictid, conflict )
            obj.conflicts{grainid}(conflictid) = conflict;
        end
        
        function resetConflicts(obj, grainid)
        % GTGRAINCONFLICTS/RESETCONFLICTS( grainid )
        %     Reset property 'conflicts' for grainid
            obj.conflicts{grainid} = repmat(GtConflict(),0,1);
        end
        
        function resetConflictsToN(obj, grainid, N)
        % GTGRAINCONFLICTS/RESETCONFLICTSTON( grainid, N )
        %     Reset property 'conflicts' to N-array for grainid
            obj.conflicts{grainid} = repmat(GtConflict(),N,1);
        end
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Getter Functions
        
        function grain = getGrain(obj, grainid)
        % GTGRAINCONFLICTS/GETGRAIN( grainid )
            grain = obj.grain{grainid};
        end

        % analyser fields
        function tot = getTotSpots(obj, grainid)
        % GTGRAINCONFLICTS/GETTOTSPOTS( grainid )
            tot = obj.analyser{grainid}.totspots;
        end

        function idspot = getIdSpot(obj, grainid)
        % GTGRAINCONFLICTS/GETIDSPOT( grainid )
            idspot = obj.analyser{grainid}.idspot;
        end

        function indexes = getIdSpotRow(obj, grainid, row)
        % GTGRAINCONFLICTS/GETIDSPOTROW( grainid, row )
            indexes = obj.analyser{grainid}.idspot(row,:);
        end

        function num = getTotGrains(obj, grainid)
        % GTGRAINCONFLICTS/GETTOTGRAINS( grainid )
            num = obj.analyser{grainid}.totgrains;
        end
        
        function id = getIdConflict(obj, grainid)
        % GTGRAINCONFLICTS/GETIDCONFLICT( grainid )
            id = obj.analyser{grainid}.idconflict;
        end
        
        function f = getFlag(obj, grainid)
        % GTGRAINCONFLICTS/GETFLAG( grainid )
            f = obj.analyser{grainid}.flag;
        end
        
        function ids = getDifspotID(obj, grainid)
        % GTGRAINCONFLICTS/GETDIFSPOTID( grainid )
            ids = obj.analyser{grainid}.difspotID;
        end

        function analyser = getAnalyser(obj, grainid)
        % GTGRAINCONFLICTS/GETANALYSER( grainid )
            analyser = obj.analyser{grainid};
        end

        % conflicts fields
        function spots = getDifspots(obj, grainid, conflictid)
        % GTGRAINCONFLICTS/GETDIFSPOTS( grainid, conflictid )
            spots = obj.conflicts{grainid}(conflictid).difspots;
        end
        
        function ids = getGrainDifspotsConflict(obj, grainid, idconflict)
        % GTGRAINCONFLICTS/GETGRAINDIFSPOTSCONFLICT( grainid, idconflict )
            list = obj.getIdConflict(grainid);
            for ii =1 : numel(list)
                tmp = obj.getDifspots(grainid, ii);
                if tmp{1} == idconflict
                    ids = tmp{2};
                end
            end
        end

        function max = getMaxNum(obj, grainid, conflictid)
        % GTGRAINCONFLICTS/GETMAXNUM( grainid, conflictid )
            max = obj.conflicts{grainid}(conflictid).maxnum;
        end

        function m = getMerge(obj, grainid, conflictid)
        % GTGRAINCONFLICTS/GETMERGE( grainid, conflictid )
            m = obj.conflicts{grainid}(conflictid).merge;
        end

        function alleq = getAllEquals(obj, grainid, conflictid)
        % GTGRAINCONFLICTS/GETALLEQUALS( grainid, conflictid )
            alleq = obj.conflicts{grainid}(conflictid).allequals;
        end
        
        function del = getDelete(obj, grainid, conflictid)
        % GTGRAINCONFLICTS/GETDELETE( grainid, conflictid )
            del = obj.conflicts{grainid}(conflictid).delete;
        end

        function conflict = getConflictId(obj, grainid, conflictid)
        % GTGRAINCONFLICTS/GETCONFLICTID( grainid, conflictid )
            conflict = obj.conflicts{grainid}(conflictid);
        end
        
        function conflict = getConflicts(obj, grainid)
        % GTGRAINCONFLICTS/GETCONFLICTS( grainid, conflictid )
            conflict = obj.conflicts{grainid};
        end


        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Analysis Functions

        function [allvalues, grainid] = getAllValuesFromCellstrField(obj, cellstr, field)
        % GTGRAINCONFLICTS/GETALLVALUESFROMCELLSTRFIELD( cellstr, field )
        %     output are 'allvalues' and 'grainid'
        %     cellstr can be 'grain'/'analyser'/'conflicts'

            if ~any(strcmp(cellstr,'conflicts') || strcmp(cellstr,'analyser') || strcmp(cellstr,'grain'))
                gtError('GtGrainConflicts:wrongArgument','Cellstr is wrong...Quitting')
            end
            cellArray = obj.(cellstr);
            if isa(cellArray,'GtAnalyser')
                cellArray = [cellArray{:}];
            elseif strcmp(cellstr,'grain')
                allvalues = [cellArray{:}];
                grainid = unique(allvalues(1,:));
                return
            end
            allvalues = [];
            grainid   = [];
            count = 0;
            sizeA = [];
            for i = 1 : length(cellArray)
              if iscell(cellArray)
                array = cellArray{i};
              elseif isa(cellArray,'GtAnalyser')
                array = cellArray(i);
              end
                sizeA = [sizeA, length(array)];
                for j = 1 : length(array)
                    current = array(j);
                    if ~isprop(current, field)
                        gtError('GtGrainConflicts:wrongArgument','Wrong property for the given cell structure...Quitting')
                    end
                    if isempty(current.(field))
                        count = count + 1;
                    else
                        grainid = [grainid; i];
                        allvalues = [allvalues; {current.(field)}];
                    end
                    fprintf('grainid: %04d size: %2d currentID: %2d isempty: %4d total: %4d length: %4d \n', ...
                        i, size(array,1) , j, count, size(allvalues, 1), size(grainid,1))
                end
            end
            sizeA = cumsum(sizeA);
            disp(['empty: ' num2str(count)])
        end

        function grainIDs = getAllFieldsValueWithQuery(obj, cellstr, field, value, query)
        % GTGRAINCONFLICTS/GETALLFIELDSVALUEWITHQUERY( cellstr, field, value, query )
        %     cellstr can be 'grain'/'analyser'/'conflicts'
        %     output is 'grainIDs'

            [allvalues, ids] = getAllValuesFromCellstrField(obj, cellstr, field);
            allvalues = [allvalues{:}];
            if query == 0
                grainIDs = ids(allvalues == value);
            elseif query == -1
                grainIDs = ids(allvalues < value);
            elseif query == 1
                grainIDs = ids(allvalues > value);
            else
                gtError('GtGrainConflicts:wrongArgument','The query is not recnognized...Quitting')
            end
        end

        function inconflict = getHasConflictsTrue(obj)
        % GTGRAINCONFLICTS/GETHASCONFLICTSTRUE()
        %     Gets only selected grains with conflicts
        %     output is 'inconflict'
            grainIDs = [obj.grain{:}];
            inconflict = unique(grainIDs(1,:));
        end
       
        function buildIdSpot(obj, grainid)
        % GTGRAINCONfLICTS/BUILDIDSPOT( grainid )
        
            difspotID = obj.getDifspotID(grainid)'; % row vector
            if isempty(difspotID)
                disp(['Warning: difspotID field is empty for grain #' grainid])
            end
            idspot = obj.getIdSpotRow(grainid,1); % row vector
            if ~isempty(idspot)
                if size(idspot,1) < size(idspot,2)
                    idspot = idspot(1,:);
                else
                    idspot = idspot(:,1);
                end
                if ~isempty(difspotID)
                    if size(difspotID,1) ~= size(idspot,1)
                        idspot = idspot';
                    end
                    [spots, ind] = intersect(difspotID, idspot, 'stable');
                    obj.setIdSpotRow(grainid, 1, spots);
                    obj.setIdSpotRow(grainid, 2, ind);
                end
            else
                obj.setIdSpot(grainid, []);
            end
        end
        
        function compareGrains(obj, grainid, conflictids, maxnum)
        % GTGRAINCONFLICTS/COMPAREGRAINS( grainid, conflictids, maxnum )

            for kk=1:numel(conflictids)
                spots  = obj.getDifspotsConflicts(grainid, conflictids(kk));
                if ~isempty(spots)
                    obj.setFlag(grainid, 1);
                    [~,current_inds,~] = intersect(obj.getDifspotID(grainid),spots,'stable');
                    obj.setDifspots(grainid, kk, {conflictids(kk), current_inds'});
                    obj.setIdSpot(grainid, []);
                    obj.setIdSpotRow(grainid, 1, spots);

                    if length(spots) > maxnum
                      obj.setMaxNum(grainid, kk, true);
                      obj.setMerge(grainid, kk, conflictids(kk));
                    end
                    
                    orig_spots = obj.getDifspotsConflicts(conflictids(kk), grainid);
                    if isempty(setdiff(orig_spots, spots))
                      obj.setAllEquals(grainid, kk, true);
                      obj.setDelete(grainid, kk, current_inds');
                    end
                else
                    obj.setConflictId(grainid, kk, []);
                end
            end

        end % end compareGrains

        function difspots = getDifspotsConflicts(obj, grainid, conflictid)
        % GTGRAINCONFLICTS/GETDIFSPOTSCONFLICTS( grainid, conflictid )
        %     output is 'difspots'
            tmp = obj.getGrain(grainid);
            difspots = tmp(2,tmp(1,:)==conflictid);
        end % end getDifspotsConflicts

        
        function deselectDifspotsFromGrain(obj, grainIDs_mod, idspots, conflictIDs)
        % GTGRAINCONFLICTS/DESELECTDIFSPOTSFROMGRAIN  Given a list of modified
        % grains and a list of difspots to be deselected, update .grain,
        % .analyser and .conflicts, basing on conflictIDs list
        
            % remove difspots from grain{} for grainIDs_mod and conflictIDs
            tmp = obj.getGrain(grainIDs_mod);
            tmp(:,tmp(1,:)==conflictIDs & ismember(tmp(2,:),idspots))=[];
            obj.setGrain(grainIDs_mod, tmp);
            
            tmp2 = obj.getGrain(conflictIDs);
            tmp2(:,tmp2(1,:)==grainIDs_mod & ismember(tmp2(2,:),idspots))=[];
            obj.setGrain(conflictIDs, tmp2);

            obj.compareGrains(grainIDs_mod, conflictIDs, []);
            
            if isempty(obj.getGrain(grainIDs_mod))
                obj.resetAnalyser(grainIDs_mod);
                obj.resetConflicts(grainIDs_mod);
            else
                obj.buildIdSpot(grainIDs_mod);
                obj.setTotSpots(grainIDs_mod, size(tmp,2));
                obj.setIdConflict(grainIDs_mod, unique(tmp(1,:)));
                obj.setTotGrains(grainIDs_mod, length(obj.getIdConflict(grainIDs_mod)));
            end
            if isempty(obj.getGrain(conflictIDs))
                obj.resetAnalyser(conflictIDs);
                obj.resetConflicts(conflictIDs);
            else
                obj.buildIdSpot(conflictIDs);
                obj.setTotSpots(conflictIDs, size(tmp2,2));
                obj.setIdConflict(conflictIDs, unique(tmp2(1,:)));
                obj.setTotGrains(conflictIDs, length(obj.getIdConflict(conflictIDs)));
            end
            
        end
        
    end % end methods
end % end classdef
