function [labXYZ, sam2lab] = gtGeoSam2Lab(samXYZ, om, labgeo, samgeo, ...
                             freevec, rot_flag)
% GTGEOSAM2LAB  SAMPLE -> LAB reference coordinate transformation.
%
% [labXYZ, sam2lab] = gtGeoSam2Lab(samXYZ, om, labgeo, samgeo, ...
%                              freevec, rot_flag)
%
% ----------------------------------------------------------------------- 
%
% Given the SAMPLE coordinates of a set of points, transforms them 
% into LAB coordinates according to an omega rotation. Omega is usually
% positive.
% It optionally creates the transformation matrix.
%
% INPUT  
%   samXYZ         - coordinates in the Sample reference in no. of voxels (n,3)
%   om             - omega angles in degrees (n,1)
%   labgeo.rotpos  - rotation axis position in Lab coordinates (1,3)
%   labgeo.rotdir  - rotation axis direction (right-handed rotation)
%   samgeo.orig    - origin of the sample reference in Lab coordinates (1,3)
%   samgeo.voxsize - [x,y,z] voxel sizes in lab units
%   samgeo.sam2lab - optional (can be left missing or empty); transformation
%                    matrix from Sample to Lab coordinates for row vectors
%                    (3,3)
%   freevec        - 0 for fixed vectors (e.g. location);
%                    1 for free vectors (e.g. orientation)
%   rot_flag       - if true, only applies rotation and the norm of the 
%                    input vectors will be unchanged; i.e. no scaling 
%                    applied according to the voxel sizes (only works if 
%                    'samgeo.sam2lab' is undefined)
%
% OUTPUT 
%   labXYZ         - row vector of SAMPLE coordinates (n,3)
%   sam2lab        - coordinate transformation matrix (field added if it 
%                    does not yet exists)
%


n = size(samXYZ,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Change of reference from Sample to Lab
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Sample to Lab coordinates:
%   (Lab is an orthonormal basis by definition.)
if (~isfield(samgeo,'sam2lab') || (isfield(samgeo,'sam2lab') && isempty(samgeo.sam2lab)))
    
    if rot_flag
        % Keep norm of input vectors -> keep unit lengths in the rows of
        % the transformation matrix
        sam2lab = [samgeo.dirx; samgeo.diry; samgeo.dirz];
    else
        sam2lab = [samgeo.dirx*samgeo.voxsize(1);...
                   samgeo.diry*samgeo.voxsize(2);...
                   samgeo.dirz*samgeo.voxsize(3)];
    end
    
elseif rot_flag
    error('Transformation matrix already defined. Set ''rot_flag'' to false.')
else
    sam2lab = samgeo.sam2lab;
end

%%%%%%%%%%%%%%%%%%%%%
%% Rotation in Lab
%%%%%%%%%%%%%%%%%%%%%

% Tensor which rotates with omega from Sample to Lab
% (omega is usually positive):
rotcomp    = gtMathsRotationMatrixComp(labgeo.rotdir, 'row');
rottensors = gtMathsRotationTensor(om, rotcomp);

if freevec    % free vectors

    % Vectors in Lab reference at the origin before rotation
    labXYZprerot = samXYZ*sam2lab;

    % Multiply element-wise to avoid loop
    pro = reshape([labXYZprerot, labXYZprerot, labXYZprerot]',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose
    labXYZ = reshape(sum(pro,1),3,n)';
    
else % offset vectors

    % Vectors relative to rot. axis position in Lab reference
    vv           = samgeo.orig - labgeo.rotpos;
    labXYZprerot = samXYZ*sam2lab + vv(ones(n,1),:);
   
    % Apply rottensor element-wise to avoid loop
    pro = reshape([labXYZprerot, labXYZprerot, labXYZprerot]',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose, set offset back
    labXYZ = reshape(sum(pro,1),3,n)' + labgeo.rotpos(ones(n,1),:);
     
end


end % of function
